@@ -28,25 +28,30 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2010 Consorcio Fernando de los Rios."
 __license__   = "LGPL"
 
-import dbus
-import os
 import imp
-from dbus.mainloop.glib import DBusGMainLoop
-from json import load
+import importlib
+import os
+from gi.repository import Gio, GLib
 
-import debug
-from keybindings import KeyBinding
-import settings
-import pronunciation_dict
+from . import debug
+from . import script_manager
+from . import settings
+from . import pronunciation_dict
+from .keybindings import KeyBinding
 
 try:
-    _dbusLoop = DBusGMainLoop()
-    _bus = dbus.SessionBus(mainloop=_dbusLoop)
-    _proxy = _bus.get_object("org.a11y.Bus", "/org/a11y/bus")
-    _desktopProps = \
-        dbus.Interface(_proxy, dbus_interface='org.freedesktop.DBus.Properties')
+    _proxy = Gio.DBusProxy.new_for_bus_sync(
+        Gio.BusType.SESSION,
+        Gio.DBusProxyFlags.NONE,
+        None,
+        'org.a11y.Bus',
+        '/org/a11y/bus',
+        'org.freedesktop.DBus.Properties',
+        None)
 except:
-    _desktopProps = None
+    _proxy = None
+
+_scriptManager = script_manager.getManager()
 
 class SettingsManager(object):
     """Settings backend manager. This class manages orca user's settings
@@ -72,6 +77,7 @@ class SettingsManager(object):
         self._backend = None
         self.profile = None
         self.backendName = backend
+        self._prefsDir = None
 
         # Dictionaries for store the default values
         # The keys and values are defined at orca.settings
@@ -98,13 +104,26 @@ class SettingsManager(object):
         self.keybindings = {}
 
         if not self._loadBackend():
-            raise Exception('SettingsManager::_loadBackend fails.')
+            raise Exception('SettingsManager._loadBackend failed.')
 
         self.customizedSettings = {}
         self._customizationCompleted = False
 
+        # For handling the currently-"classic" application settings
+        self.settingsPackages = ["app-settings"]
+        self._knownAppSettings = {}
+
+        debug.println(debug.LEVEL_FINEST, 'INFO: Settings manager initialized')
+
+    def activate(self, prefsDir=None, customSettings={}):
+        debug.println(debug.LEVEL_FINEST, 'INFO: Activating settings manager')
+
+        self.customizedSettings.update(customSettings)
+        self._prefsDir = prefsDir \
+            or os.path.join(GLib.get_user_data_dir(), "orca")
+
         # Load the backend and the default values
-        self._backend = self.backendModule.Backend()
+        self._backend = self.backendModule.Backend(self._prefsDir)
         self._setDefaultGeneral()
         self._setDefaultPronunciations()
         self._setDefaultKeybindings()
@@ -119,27 +138,21 @@ class SettingsManager(object):
         # yet, so we need to create the user config directories and store the
         # initial default settings
         #
-        if self.isFirstStart():
-            self._createDefaults()
+        self._createDefaults()
+
+        debug.println(debug.LEVEL_FINEST, 'INFO: Settings manager activated')
 
         # Set the active profile and load its stored settings
         if self.profile is None:
             self.profile = self.general.get('startingProfile')[1]
         self.setProfile(self.profile)
 
-        # For handling the currently-"classic" application settings
-        self.settingsPackages = ["app-settings"]
-        self._knownAppSettings = {}
-
-        debug.println(debug.LEVEL_FINEST, 'INFO: Settings manager initialized')
-
     def _loadBackend(self):
         """Load specific backend for manage user settings"""
 
         try:
-            backend = 'backends.%s_backend' % self.backendName
-            self.backendModule = __import__(backend, globals(), locals(),
-                                            backend, -1)
+            backend = '.backends.%s_backend' % self.backendName
+            self.backendModule = importlib.import_module(backend, 'orca')
             return True
         except:
             return False
@@ -155,7 +168,7 @@ class SettingsManager(object):
         # Set up the user's preferences directory
         # ($XDG_DATA_HOME/orca by default).
         #
-        orcaDir = settings.userPrefsDir
+        orcaDir = self._prefsDir
         _createDir(orcaDir)
 
         # Set up $XDG_DATA_HOME/orca/orca-scripts as a Python package
@@ -164,7 +177,7 @@ class SettingsManager(object):
         _createDir(orcaScriptDir)
         initFile = os.path.join(orcaScriptDir, "__init__.py")
         if not os.path.exists(initFile):
-            os.close(os.open(initFile, os.O_CREAT, 0700))
+            os.close(os.open(initFile, os.O_CREAT, 0o700))
 
         # Set up $XDG_DATA_HOME/orca/app-settings as a Python package.
         #
@@ -172,22 +185,22 @@ class SettingsManager(object):
         _createDir(orcaSettingsDir)
         initFile = os.path.join(orcaSettingsDir, "__init__.py")
         if not os.path.exists(initFile):
-            os.close(os.open(initFile, os.O_CREAT, 0700))
+            os.close(os.open(initFile, os.O_CREAT, 0o700))
 
         # Set up $XDG_DATA_HOME/orca/orca-customizations.py empty file and
         # define orcaDir as a Python package.
         initFile = os.path.join(orcaDir, "__init__.py")
         if not os.path.exists(initFile):
-            os.close(os.open(initFile, os.O_CREAT, 0700))
+            os.close(os.open(initFile, os.O_CREAT, 0o700))
 
         userCustomFile = os.path.join(orcaDir, "orca-customizations.py")
         if not os.path.exists(userCustomFile):
-            os.close(os.open(userCustomFile, os.O_CREAT, 0700))
+            os.close(os.open(userCustomFile, os.O_CREAT, 0o700))
 
-
-        self._backend.saveDefaultSettings(self.defaultGeneral,
-                                          self.defaultPronunciations,
-                                          self.defaultKeybindings)
+        if self.isFirstStart():
+            self._backend.saveDefaultSettings(self.defaultGeneral,
+                                              self.defaultPronunciations,
+                                              self.defaultKeybindings)
 
     def _setDefaultPronunciations(self):
         """Get the pronunciations by default from orca.settings"""
@@ -215,12 +228,12 @@ class SettingsManager(object):
             return self.customizedSettings
 
         originalSettings = {}
-        for key, value in settings.__dict__.items():
+        for key, value in list(settings.__dict__.items()):
             originalSettings[key] = value
 
         self._customizationCompleted = self._loadUserCustomizations()
 
-        for key, value in originalSettings.items():
+        for key, value in list(originalSettings.items()):
             customValue = settings.__dict__.get(key)
             if value != customValue:
                 self.customizedSettings[key] = customValue
@@ -231,7 +244,7 @@ class SettingsManager(object):
         likelihood that the results won't be different if we keep trying."""
 
         success = False
-        pathList = [settings.userPrefsDir]
+        pathList = [self._prefsDir]
         try:
             msg = "Attempt to load orca-customizations "
             (fileHnd, moduleName, desc) = \
@@ -250,6 +263,9 @@ class SettingsManager(object):
         debug.println(debug.LEVEL_ALL, msg)
         return success
 
+    def getPrefsDir(self):
+        return self._prefsDir
+
     def setSetting(self, settingName, settingValue):
         self._setSettingsRuntime({settingName:settingValue})
 
@@ -311,27 +327,27 @@ class SettingsManager(object):
         return not alreadyEnabled
 
     def isAccessibilityEnabled(self):
-        if not _desktopProps:
+        if not _proxy:
             return False
 
-        return bool(_desktopProps.Get('org.a11y.Status', 'IsEnabled'))
+        return _proxy.Get('(ss)', 'org.a11y.Status', 'IsEnabled')
 
     def setAccessibility(self, enable):
-        if not _desktopProps:
+        if not _proxy:
             return False
 
-        _desktopProps.Set('org.a11y.Status', 'IsEnabled', enable)
-        return True
+        vEnable = GLib.Variant('b', enable)
+        _proxy.Set('(ssv)', 'org.a11y.Status', 'IsEnabled', vEnable)            
 
     def isScreenReaderServiceEnabled(self):
         """Returns True if the screen reader service is enabled. Note that
         this does not necessarily mean that Orca (or any other screen reader)
         is running at the moment."""
 
-        if not _desktopProps:
+        if not _proxy:
             return False
 
-        return bool(_desktopProps.Get('org.a11y.Status', 'ScreenReaderEnabled'))
+        return _proxy.Get('(ss)', 'org.a11y.Status', 'ScreenReaderEnabled')
 
     def setStartingProfile(self, profile=None):
         if profile is None:
@@ -354,10 +370,10 @@ class SettingsManager(object):
         return (general, pronunciations, keybindings)
 
     def _setSettingsRuntime(self, settingsDict):
-        for key, value in settingsDict.items():
+        for key, value in list(settingsDict.items()):
             setattr(settings, str(key), value)
         self._getCustomizedSettings()
-        for key, value in self.customizedSettings.items():
+        for key, value in list(self.customizedSettings.items()):
             setattr(settings, str(key), value)
         self._setPronunciationsRuntime()
 
@@ -393,7 +409,7 @@ class SettingsManager(object):
         as the profile's."""
         self.profileGeneral = {}
 
-        for key, value in general.items():
+        for key, value in list(general.items()):
             if isinstance(value, unicode):
                 value = value.encode('UTF-8')
             if key in settings.excludeKeys:
@@ -453,7 +469,7 @@ class SettingsManager(object):
 
     def overrideKeyBindings(self, script, scriptKeyBindings):
         keybindingsSettings = self.getKeybindings(self.profile)
-        for handlerString, bindingTuples in keybindingsSettings.items():
+        for handlerString, bindingTuples in list(keybindingsSettings.items()):
             handler = script.inputEventHandlers.get(handlerString)
             if not handler:
                 continue
@@ -482,26 +498,6 @@ class SettingsManager(object):
 
         return self._backend.availableProfiles()
 
-
-    def importProfile(self, fileName):
-        """Import profile from a given filename"""
-
-        prefs = {}
-        with open(fileName) as settingsFile:
-            prefs = load(settingsFile)
-
-        general = {}
-
-        for key, value in prefs.items():
-            if key not in settings.excludeKeys:
-                general[key] = value
-
-        pronunciations = prefs.get('pronunciations', {})
-        keybindings = prefs.get('keybindings', {})
-
-        self.saveSettings(general, pronunciations, keybindings)
-        return True
-
     def loadAppSettings(self, script):
         """Load the users application specific settings for an app.
         Note that currently the settings manager does not manage
@@ -515,9 +511,6 @@ class SettingsManager(object):
         self._loadProfileSettings()
         script.voices = self.getSetting('voices')
 
-        import orca
-        _scriptManager = getattr(orca, '_scriptManager')
-
         app = script.app
         moduleName = _scriptManager.getModuleName(app)
         if not moduleName:
@@ -528,13 +521,13 @@ class SettingsManager(object):
             name = '.'.join((package, moduleName))
             debug.println(debug.LEVEL_FINEST, "Looking for %s.py" % name)
             try:
-                module = __import__(name, globals(), locals(), [''])
+                module = importlib.import_module(name)
             except ImportError:
                 debug.println(
                     debug.LEVEL_FINEST, "Could not import %s.py" % name)
                 continue
             try:
-                reload(module)
+                imp.reload(module)
             except:
                 debug.println(debug.LEVEL_FINEST, "Could not load %s.py" % name)
                 module = None
@@ -546,10 +539,10 @@ class SettingsManager(object):
             return
 
         self._knownAppSettings[name] = module
-        reload(self._knownAppSettings[name])
+        imp.reload(self._knownAppSettings[name])
 
         appVoices = self.getSetting('voices')
-        for voiceType, voiceDef in appVoices.items():
+        for voiceType, voiceDef in list(appVoices.items()):
             script.voices[voiceType].update(voiceDef)
 
         keybindings = getattr(module, 'overrideAppKeyBindings', None)
@@ -565,7 +558,7 @@ class SettingsManager(object):
 
 def getVoiceKey(voice):
     voicesKeys = getattr(settings, 'voicesKeys')
-    for key in voicesKeys.keys():
+    for key in list(voicesKeys.keys()):
         if voicesKeys[key] == voice:
             return key
     return ""
@@ -579,7 +572,7 @@ def getValueForKey(prefsDict, key):
 
     value = None
     if key in prefsDict:
-        if type(prefsDict[key]) is str:
+        if isinstance(prefsDict[key], str):
             if key in need2repr:
                 value = "\'%s\'" % prefsDict[key]
             elif key  == 'voices':
@@ -604,3 +597,8 @@ def getRealValues(prefs):
     #for key in prefs.keys():
     #    prefs[key] = getValueForKey(prefs, key)
     return prefs
+
+_manager = SettingsManager()
+
+def getManager():
+    return _manager
