@@ -1,18 +1,18 @@
-# Copyright 2006, 2007, 2008 Brailcom, o.p.s.
+# Copyright 2006, 2007, 2008, 2009 Brailcom, o.p.s.
 #
 # Author: Tomas Cerha <cerha@brailcom.org>
 #
 # This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Library General Public
+# modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
-# version 2 of the License, or (at your option) any later version.
+# version 2.1 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Library General Public License for more details.
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU Library General Public
+# You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the
 # Free Software Foundation, Inc., Franklin Street, Fifth Floor,
 # Boston MA  02110-1301 USA.
@@ -31,22 +31,24 @@
 NOTE: THIS IS EXPERIMENTAL ONLY AND IS NOT A SUPPORTED COMPONENT OF ORCA.
 """
 
-__id__ = "$Id: speechdispatcherfactory.py 4375 2008-12-01 19:19:15Z wwalker $"
-__version__   = "$Revision: 4375 $"
-__date__      = "$Date: 2008-12-01 14:19:15 -0500 (Mon, 01 Dec 2008) $"
+__id__ = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
 __author__    = "Tomas Cerha <cerha@brailcom.org>"
 __copyright__ = "Copyright (c) 2006-2008 Brailcom, o.p.s."
 __license__   = "LGPL"
 
-import gobject
+from gi.repository import GObject
+import re
 
-import debug
-import speechserver
-import settings
-import orca
-import orca_state
-from acss import ACSS
-from orca_i18n import _
+from . import chnames
+from . import debug
+from . import speechserver
+from . import settings
+from . import orca_state
+from . import punctuation_settings
+from .acss import ACSS
+from .orca_i18n import _
 
 try:
     import speechd
@@ -61,6 +63,9 @@ else:
     else:
         _speechd_version_ok = True
 
+PUNCTUATION = re.compile('[^\w\s]', re.UNICODE)
+ELLIPSIS = re.compile('(\342\200\246|\.\.\.\s*)')
+
 class SpeechServer(speechserver.SpeechServer):
     # See the parent class for documentation.
 
@@ -78,6 +83,7 @@ class SpeechServer(speechserver.SpeechServer):
     
     KEY_NAMES = {
         '_':     'underscore',
+        ' ':     'space',
         'space': 'space',
         '"':     'double-quote',
         }
@@ -91,10 +97,12 @@ class SpeechServer(speechserver.SpeechServer):
     getFactoryName = staticmethod(getFactoryName)
 
     def getSpeechServers():
+        servers = []
         default = SpeechServer._getSpeechServer(SpeechServer.DEFAULT_SERVER_ID)
-        servers = [default]
-        for module in default.list_output_modules():
-            servers.append(SpeechServer._getSpeechServer(module))
+        if default is not None:
+            servers.append(default)
+            for module in default.list_output_modules():
+                servers.append(SpeechServer._getSpeechServer(module))
         return servers
     getSpeechServers = staticmethod(getSpeechServers)
 
@@ -121,7 +129,7 @@ class SpeechServer(speechserver.SpeechServer):
     getSpeechServer = staticmethod(getSpeechServer)
 
     def shutdownActiveServers():
-        for server in SpeechServer._active_servers.values():
+        for server in list(SpeechServer._active_servers.values()):
             server.shutdown()
     shutdownActiveServers = staticmethod(shutdownActiveServers)
 
@@ -169,6 +177,11 @@ class SpeechServer(speechserver.SpeechServer):
         # the list will contain the names of all available "real"
         # voices provided by the speech engine.
         #
+        try:
+            serverId = serverId.encode("UTF-8")
+        except UnicodeDecodeError:
+            pass
+
         self._default_voice_name = _("%s default voice") % serverId
         
         try:
@@ -182,12 +195,18 @@ class SpeechServer(speechserver.SpeechServer):
 
     def _init(self):
         self._client = client = speechd.SSIPClient('Orca', component=self._id)
+        client.set_priority(speechd.Priority.MESSAGE)
         if self._id != self.DEFAULT_SERVER_ID:
             client.set_output_module(self._id)
         self._current_voice_properties = {}
         mode = self._PUNCTUATION_MODE_MAP[settings.verbalizePunctuationStyle]
         client.set_punctuation(mode)
-        
+
+    def updatePunctuationLevel(self):
+        """ Punctuation level changed, inform this speechServer. """
+        mode = self._PUNCTUATION_MODE_MAP[settings.verbalizePunctuationStyle]
+        self._client.set_punctuation(mode)
+
     def _send_command(self, command, *args, **kwargs):
         if hasattr(speechd, 'SSIPCommunicationError'):
             try:
@@ -215,18 +234,23 @@ class SpeechServer(speechserver.SpeechServer):
         self._send_command(self._client.set_volume, volume)
 
     def _set_family(self, acss_family):
-        locale = acss_family[speechserver.VoiceFamily.LOCALE]
-        if locale:
-            lang = locale.split('_')[0]
+        familyLocale = acss_family.get(speechserver.VoiceFamily.LOCALE)
+        if not familyLocale:
+            import locale
+            familyLocale, encoding = locale.getdefaultlocale()
+        if familyLocale:
+            lang = familyLocale.split('_')[0]
             if lang and len(lang) == 2:
-                self._send_command(self._client.set_language, lang)
+                self._send_command(self._client.set_language, str(lang))
         try:
             # This command is not available with older SD versions.
             set_synthesis_voice = self._client.set_synthesis_voice
         except AttributeError:
             pass
         else:
-            name = acss_family[speechserver.VoiceFamily.NAME]
+            name = acss_family.get(speechserver.VoiceFamily.NAME)
+            if isinstance(name, unicode):
+                name = name.encode('UTF-8')
             if name != self._default_voice_name:
                 self._send_command(set_synthesis_voice, name)
             
@@ -236,11 +260,93 @@ class SpeechServer(speechserver.SpeechServer):
         current = self._current_voice_properties
         for acss_property, method in self._acss_manipulators:
             value = acss.get(acss_property)
-            if value is not None and current.get(acss_property) != value:
-                method(value)
-                current[acss_property] = value
+            if value is not None:
+                if current.get(acss_property) != value:
+                    method(value)
+                    current[acss_property] = value
+            elif acss_property == ACSS.FAMILY and current.get(acss_property) \
+                    and acss == settings.voices[settings.DEFAULT_VOICE]:
+                # We need to explicitly reset (at least) the family.
+                # See bgo#626072.
+                #
+                method({})
+                current[acss_property] = {}
+
+    def __addVerbalizedPunctuation(self, oldText):
+        """Depending upon the users verbalized punctuation setting,
+        adjust punctuation symbols in the given text to their pronounced
+        equivalents. The pronounced text will either replace the
+        punctuation symbol or be inserted before it. In the latter case,
+        this is to retain spoken prosity.
+
+        Arguments:
+        - oldText: text to be parsed for punctuation.
+
+        Returns a text string with the punctuation symbols adjusted accordingly.
+        """
+
+        # Translators: we replace the ellipses (both manual and UTF-8)
+        # with a spoken string.  The extra space you see at the beginning
+        # is because we need the speech synthesis engine to speak the
+        # new string well.  For example, "Open..." turns into
+        # "Open dot dot dot".
+        #
+        spokenEllipsis = _(" dot dot dot") + " "
+        newText = re.sub(ELLIPSIS, spokenEllipsis, oldText)
+        try:
+            newText = newText.decode("UTF-8")
+        except UnicodeEncodeError:
+            pass
+
+        symbols = set(re.findall(PUNCTUATION, newText))
+        for symbol in symbols:
+            try:
+                level, action = punctuation_settings.getPunctuationInfo(symbol)
+            except:
+                continue
+
+            if level != punctuation_settings.LEVEL_NONE:
+                # Speech Dispatcher should handle it.
+                #
+                continue
+
+            charName = " %s " % chnames.getCharacterName(symbol)
+            if action == punctuation_settings.PUNCTUATION_INSERT:
+                charName += symbol
+            try:
+                charName = charName.decode("UTF-8")
+            except UnicodeEncodeError:
+                pass
+            newText = re.sub(symbol, charName, newText)
+
+        if orca_state.activeScript:
+            newText = orca_state.activeScript.utilities.adjustForDigits(newText)
+
+        try:
+            newText = newText.encode("UTF-8")
+        except UnicodeDecodeError:
+            pass
+
+        return newText
 
     def _speak(self, text, acss, **kwargs):
+        if isinstance(text, ACSS):
+            text = ''
+        text = self.__addVerbalizedPunctuation(text)
+        if orca_state.activeScript and orca_state.usePronunciationDictionary:
+            text = orca_state.activeScript.\
+                utilities.adjustForPronunciation(text)
+
+        # Replace no break space characters with plain spaces since some
+        # synthesizers cannot handle them.  See bug #591734.
+        #
+        text = text.decode("UTF-8").replace(u'\u00a0', ' ').encode("UTF-8")
+
+        # Replace newline followed by full stop, since
+        # this seems to crash sd, see bgo#618334.
+        #
+        text = text.replace('\n.', '\n')
+
         self._apply_acss(acss)
         self._send_command(self._client.speak, text, **kwargs)
 
@@ -251,7 +357,7 @@ class SpeechServer(speechserver.SpeechServer):
 
         """
         try:
-            context, acss = iterator.next()
+            context, acss = next(iterator)
         except StopIteration:
             pass
         else:
@@ -267,11 +373,11 @@ class SpeechServer(speechserver.SpeechServer):
                         context.currentOffset = context.startOffset
                 elif t == speechserver.SayAllContext.COMPLETED:
                     context.currentOffset = context.endOffset
-                gobject.idle_add(orca_callback, context, t)
+                GObject.idle_add(orca_callback, context, t)
                 if t == speechserver.SayAllContext.COMPLETED:
-                    gobject.idle_add(self._say_all, iterator, orca_callback)
+                    GObject.idle_add(self._say_all, iterator, orca_callback)
             self._speak(context.utterance, acss, callback=callback,
-                        event_types=self._CALLBACK_TYPE_MAP.keys())
+                        event_types=list(self._CALLBACK_TYPE_MAP.keys()))
         return False # to indicate, that we don't want to be called again.
 
     def _cancel(self):
@@ -280,7 +386,10 @@ class SpeechServer(speechserver.SpeechServer):
     def _change_default_speech_rate(self, decrease=False):
         acss = settings.voices[settings.DEFAULT_VOICE]
         delta = settings.speechRateDelta * (decrease and -1 or +1)
-        rate = acss[ACSS.RATE]
+        try:
+            rate = acss[ACSS.RATE]
+        except KeyError:
+            rate = 50
         acss[ACSS.RATE] = max(0, min(99, rate + delta))
         debug.println(debug.LEVEL_CONFIGURATION,
                       "Speech rate is now %d" % rate)
@@ -290,7 +399,10 @@ class SpeechServer(speechserver.SpeechServer):
     def _change_default_speech_pitch(self, decrease=False):
         acss = settings.voices[settings.DEFAULT_VOICE]
         delta = settings.speechPitchDelta * (decrease and -1 or +1)
-        pitch = acss[ACSS.AVERAGE_PITCH]
+        try:
+            pitch = acss[ACSS.AVERAGE_PITCH]
+        except KeyError:
+            pitch = 5
         acss[ACSS.AVERAGE_PITCH] = max(0, min(9, pitch + delta))
         debug.println(debug.LEVEL_CONFIGURATION,
                       "Speech pitch is now %d" % pitch)
@@ -327,8 +439,7 @@ class SpeechServer(speechserver.SpeechServer):
     def speak(self, text=None, acss=None, interrupt=True):
         #if interrupt:
         #    self._cancel()
-        if orca_state.activeScript and orca_state.usePronunciationDictionary:
-            text = orca_state.activeScript.adjustForPronunciation(text)
+
         if text:
             self._speak(text, acss)
 
@@ -344,30 +455,38 @@ class SpeechServer(speechserver.SpeechServer):
                 self._speak(utterance, acss)
 
     def sayAll(self, utteranceIterator, progressCallback):
-        gobject.idle_add(self._say_all, utteranceIterator, progressCallback)
+        GObject.idle_add(self._say_all, utteranceIterator, progressCallback)
 
     def speakCharacter(self, character, acss=None):
         self._apply_acss(acss)
         if character == '\n':
             self._send_command(self._client.sound_icon, 'end-of-line')
-        else:
-            self._send_command(self._client.char, character)
+            return
 
-    def speakKeyEvent(self, event_string, eventType):
-        if eventType == orca.KeyEventType.PRINTABLE:
+        name = chnames.getCharacterName(character)
+        if not name:
+            self._send_command(self._client.char, character)
+            return
+
+        if orca_state.activeScript and orca_state.usePronunciationDictionary:
+            name = orca_state.activeScript.\
+                utilities.adjustForPronunciation(name)
+        self.speak(name, acss)
+
+    def speakKeyEvent(self, event):
+        if event.isPrintableKey():
             # We currently only handle printable characters by Speech
             # Dispatcher's KEY command.  For other keys, such as Ctrl, Shift
             # etc. we prefer Orca's verbalization.
-            if event_string.decode("UTF-8").isupper():
+            if event.event_string.decode("UTF-8").isupper():
                 acss = settings.voices[settings.UPPERCASE_VOICE]
             else:
                 acss = None
-            key = self.KEY_NAMES.get(event_string, event_string)
+            key = self.KEY_NAMES.get(event.event_string, event.event_string)
             self._apply_acss(acss)
             self._send_command(self._client.key, key)
         else:
-            return super(SpeechServer, self).speakKeyEvent(event_string, 
-                                                           eventType)
+            return super(SpeechServer, self).speakKeyEvent(event)
 
     def increaseSpeechRate(self, step=5):
         self._change_default_speech_rate()
